from pathlib import Path

DOCS = "https://culebront-pipeline.readthedocs.io/en/latest/"
GIT_URL = "https://forge.ird.fr/diade/culebront_pipeline"

SINGULARITY_URL_FILES = [('oras://registry.forge.ird.fr/diade/culebront_pipeline/apptainer/apptainer.culebront_tools.sif:0.0.1',
                          f'INSTALL_PATH/containers/apptainer.culebront_tools.sif')
                         ]

DATATEST_URL_FILES = ("https://itrop.ird.fr/culebront_utilities/Data-Xoo-sub.zip", "Data-Xoo-sub.zip")


AVAIL_ASSEMBLY = ("CANU", "FLYE", "MINIASM", "RAVEN", "SMARTDENOVO", "SHASTA")
AVAIL_CORRECTION = ("MEDAKA", "PILON")
AVAIL_POLISHING = ("RACON")
AVAIL_QUALITY = ("BUSCO", "QUAST", "BLOBTOOLS", "ASSEMBLYTICS", "FLAGSTATS", "MERQURY")

ALLOW_FASTQ_EXT = (".fastq", ".fq", ".fq.gz", ".fastq.gz")




#!/usr/bin/env python3

import nbformat as nbf
from pathlib import Path
from datetime import datetime

nb = nbf.v4.new_notebook()
nb['cells'] = []

date_time = datetime.now()


jupyter = Path(snakemake.output.qmd_by_sample)
name_jupyter = jupyter.stem
dir_jupyter = jupyter.parent
jupyter = jupyter.resolve().as_posix()

header = """---
format: 
  html:
    code-fold: false
    toc: true
    toc-depth: 4
    toc-expand: 2
jupyter: python3
execute: 
  echo: false
  warning: false
  message: false
  error: false
  cache: false
  output: asis
filters:
   - lightbox
lightbox:
  match: auto
  effect: fade
  loop: false
---
"""

nb['cells'].append(nbf.v4.new_markdown_cell(header))

import_packages = """
import pandas as pd
from pathlib import Path
from itables import init_notebook_mode, show
import itables.options as opt
opt.columnDefs = [{"className": "dt-center", "targets": "_all"}]
opt.style = "table-layout:auto;width:auto;margin:auto;caption-side:bottom;float:left"
opt.showIndex = False
init_notebook_mode(all_interactive=False)
# round to two decimal places in python pandas 
pd.options.display.float_format = '{:.2f}'.format
"""

nb['cells'].append(nbf.v4.new_code_cell(import_packages))

# START REPORT

text = f"""
# {snakemake.params.sample_name}
"""

nb['cells'].append(nbf.v4.new_markdown_cell(text))

# BENCHMARK

title_benchmark = f"""
## Benchmark time

Time used (in seconds) to execute activated tools in this pipeline.  

"""

nb['cells'].append(nbf.v4.new_markdown_cell(title_benchmark))

benchmark_table = f"""

# create a sample DataFrame
df = pd.read_csv('{snakemake.input.bench_list[0]}', index_col=[0,1], header=0)
df.sort_values(by=df.columns[0], axis=0, inplace=True)

# print the DataFrame
S = df.style.format(precision=2)
S.background_gradient(axis=None, cmap="YlOrRd")
opt.showIndex = True
show(S, dom="lfr", paging=False, classes="display cell-border", scrollCollapse=True, scrollX=True)
opt.showIndex = False
"""

nb['cells'].append(nbf.v4.new_code_cell(benchmark_table))

# STAT ASSEMBLY

title_stat = f"""
## Statistics

Useful global statistics to compare assemblies obtained at each step of this pipeline for the whole of samples.
- **Sequence_count**  is the number of contigs. 
- **GC content** is the percentage of GC nucleic bases.
- **Longest** is the longest contig assembled.
- **Shortest** is the shortest contig assembled.
- **Mean** : is the sum of the lenght of contigs divised by the number of contigs.
- **Median** : the median is the value separating the higher half from the lower half of contigs.
- **Total_bps** : Total bases number into contigs.
- **Ln** : n=10,n=20,n=30, n=40,n=50 : minimun number of contigs that produces n % of the bases
- **Nn** : n=10,n=20,n=30, n=40,n=50 : length of the contig for which longer length contigs cover at least n % of the assembly

"""

nb['cells'].append(nbf.v4.new_markdown_cell(title_stat))

stat_table = f"""
# create a sample DataFrame
df = pd.read_csv('{snakemake.input.stats_assembly[0]}', index_col=False, header=0)
df.sort_values(by=df.columns[0], axis=0, inplace=True)

# print the DataFrame
S = df.style.format(precision=2)
S.background_gradient(axis=0, cmap="coolwarm")
show(S, dom="lftr",paging=False, classes="display compact cell-border", scrollCollapse=True, scrollX=True)


"""

nb['cells'].append(nbf.v4.new_code_cell(stat_table))

# BUSCO

if "busco_stats" in snakemake.input.keys():
    title_busco = f"""
## BUSCO

BUSCO sets are collections of orthologous groups with near-universally-distributed single-copy genes in each species
    """

    nb['cells'].append(nbf.v4.new_markdown_cell(title_busco))

    busco_table = f"""
    # create a sample DataFrame
    df = pd.read_csv('{snakemake.input.busco_stats[0]}', index_col=False, header=0)
    df.sort_values(by=df.columns[0], axis=0, inplace=True)

    # print the DataFrame
    S = df.style.format(precision=2)
    S.background_gradient(axis=0, cmap="YlOrRd")
    show(S, dom="lfr", paging=False, classes="display compact cell-border", scrollCollapse=True, scrollX=True)


    """

    nb['cells'].append(nbf.v4.new_code_cell(busco_table))


## BLOBTOOLS
if "blob_files" in snakemake.input.keys():
    blobresult = """
## Blobtools
    
    """
    assembler_found = []
    for blob_file in snakemake.input.blob_files:
        assembler = Path(f"{blob_file}").stem.split("_")[1]
        quality_step = '_'.join(Path(f"{blob_file}").stem.split("_")[2:])
        path = Path(f"{blob_file}").parent
        read_cov = path.joinpath("read_cov.png").as_posix()
        blob = path.joinpath("blob.png").as_posix()
        newblob = dir_jupyter.joinpath(f'blob_{assembler}_{quality_step}.png')
        newreadcov = dir_jupyter.joinpath(f'read_cov_{assembler}_{quality_step}.png')
        if not newblob.exists():
            newblob.symlink_to(blob)
        if not newreadcov.exists():
            newreadcov.symlink_to(read_cov)
        if assembler not in assembler_found:
            blobresult = f"{blobresult}\n### {assembler} \n\n"
            assembler_found.append(assembler)
        blobresult = f"{blobresult}\n\n\n#### {quality_step}\n\n"
        blobresult = f"""{blobresult}
![thumbnail]({newblob.name}){{width='48%' group='{newblob.name}'}}
![thumbnail]({newreadcov.name}){{width='48%' group='{newreadcov.name}'}}\n\n"""


    nb['cells'].append(nbf.v4.new_markdown_cell(blobresult))

## ASSEMBLYTICS
if "assemblytics_files" in snakemake.input.keys():
    assemblyticsresults = """
## Assemblytics
    
blablabla assemblytics blabla rules ...pouet 
    """
    zip_list = zip(snakemake.input.assemblytics_dotplot, snakemake.input.assemblytics_Nchart, snakemake.input.assemblytics_log_all_sizes)
    assembler_found = []
    for dotplot, nchart, log_size in zip_list:
        assembler = Path(f"{dotplot}").stem.split("__")[1]
        quality_step = '_'.join(Path(f"{dotplot}").stem.split("__")[2:])
        quality_step = quality_step.split(".")[0]
        
        newdotplot = dir_jupyter.joinpath(Path(f"{dotplot}").name)
        if not newdotplot.exists():
            newdotplot.symlink_to(dotplot)
        newNchart = dir_jupyter.joinpath(Path(f"{nchart}").name)
        if not newNchart.exists():
            newNchart.symlink_to(nchart)
        newLogsize = dir_jupyter.joinpath(Path(f"{log_size}").name)
        if not newLogsize.exists():
            newLogsize.symlink_to(log_size)
        
        if assembler not in assembler_found:
            assemblyticsresults = f"{assemblyticsresults}\n### {assembler} \n\n"
            assembler_found.append(assembler)
        assemblyticsresults = f"{assemblyticsresults}\n\n\n#### {quality_step}\n\n"
        assemblyticsresults = f"""
{assemblyticsresults}\n
![thumbnail]({newdotplot.name}){{width='48%' group='{newdotplot.name}'}}
![thumbnail]({newNchart.name}){{width='48%' group='{newdotplot.name}'}}
![thumbnail]({newLogsize.name}){{width='48%' group='{newdotplot.name}'}}
![thumbnail]({newreadcov.name}){{width='48%' group='{newdotplot.name}'}}\n\n"""

    nb['cells'].append(nbf.v4.new_markdown_cell(assemblyticsresults))

# FLAGSTATS

if "flagstats_stats" in snakemake.input.keys():
    title_flagstat = f"""
## Flagstats

SAMTOOLS FLAGSTATS was used to calculate remapping between illumina reads and each last assembler.
    """

    nb['cells'].append(nbf.v4.new_markdown_cell(title_flagstat))

    flagstat_table = f"""
    # create a sample DataFrame
    df = pd.read_csv('{snakemake.input.flagstats_stats[0]}', index_col=False, header=0)
    df.sort_values(by=df.columns[0], axis=0, inplace=True)

    # print the DataFrame
    S = df.style.format(precision=2)
    show(S, dom="lfr", paging=False, classes="display compact cell-border", scrollCollapse=True, scrollX=True)

    """

    nb['cells'].append(nbf.v4.new_code_cell(flagstat_table))


# MERQURY

if "merqury_files" in snakemake.input.keys():
    title_merqury = f"""
## MERQURY

MERQURY : Overall k-mer evaluation is performed using the k-mer multiplicity of illumina WGS reads
    """

    nb['cells'].append(nbf.v4.new_markdown_cell(title_merqury))

    merqury_table = f"""
    # create a sample DataFrame
    df = pd.read_csv('{snakemake.input.merqury_files[0]}', index_col=False, sep="\t", header=None, names=['Assembly','mapped_kmer', 'all_kmers', 'QV', 'error_rate'])
    df.sort_values(by=df.columns[0], axis=0, inplace=True)

    # print the DataFrame
    S = df.style.format(precision=2)
    show(S, dom="r", paging=False, classes="display compact cell-border", scrollCollapse=True, scrollX=True)


    """

    nb['cells'].append(nbf.v4.new_code_cell(merqury_table))

if "quast_file" in snakemake.input.keys():
    title_quast = f"""
## QUAST

QUAST is a good starting point to help evaluate the quality of assemblies. It provides many helpful contiguity statistics.
    """

    nb['cells'].append(nbf.v4.new_markdown_cell(title_quast))

    quast_path_file = Path(snakemake.input.quast_file[0])
    new_quast_file = dir_jupyter.joinpath(quast_path_file.name)
    if not new_quast_file.exists():
        new_quast_file.symlink_to(quast_path_file)
    quast_link = f'''
{quast_path_file}
{new_quast_file}

[Open QUAST report on new window ...]({new_quast_file.name}){{target="_blank"}}
    '''

    nb['cells'].append(nbf.v4.new_markdown_cell(quast_link))

## write to file

with open(jupyter, 'w') as f:
    nbf.write(nb, f)



#!/usr/bin/env python3

import nbformat as nbf
from pathlib import Path
from datetime import datetime

nb = nbf.v4.new_notebook()
nb['cells'] = []

date_time = datetime.now()


jupyter = Path(snakemake.output.about_ipynb)
name_jupyter = jupyter.stem
dir_jupyter = jupyter.parent
jupyter = jupyter.resolve().as_posix()

header = """---
format: 
  html:
    code-fold: false
    toc: true
    toc-depth: 3
    toc-expand: 2
jupyter: python3
execute: 
  echo: false
  warning: false
  message: false
  error: false
  cache: false
---
"""

nb['cells'].append(nbf.v4.new_markdown_cell(header))

text = """
# About Workflow 

## Version of tools used
"""

code_version = f"""
import pandas as pd
from pathlib import Path
from IPython.display import display, Markdown

# create a sample DataFrame
df = pd.read_csv('{snakemake.params.versions}', index_col=False)
df.sort_values(by=df.columns[0], axis=0, inplace=True)

# print the DataFrame
Markdown(df.to_markdown(index=False))
"""
nb['cells'].append(nbf.v4.new_markdown_cell(text))

nb['cells'].append(nbf.v4.new_code_cell(code_version))

text = f"""
## Rulegraph

![dag_file]({Path(snakemake.input.dag).name}){{.center}}

## Config file Parameters

```yaml
{snakemake.params.config_yaml}
```
"""

nb['cells'].append(nbf.v4.new_markdown_cell(text))



text = """
## Snakemake report

"""

code_version = f"""
from IPython.display import IFrame
IFrame('{Path(snakemake.input.report_snakemake).name}', width=850, height=500)
"""
nb['cells'].append(nbf.v4.new_markdown_cell(text))

nb['cells'].append(nbf.v4.new_code_cell(code_version))




with open(jupyter, 'w') as f:
    nbf.write(nb, f)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.

import culebrONT
import toml
import sphinx_rtd_theme
from datetime import datetime

with open('../../pyproject.toml', 'r') as f:
    conf = toml.load(f)

# The short X.Y version.
version = culebrONT.__version__
# The full version, including alpha/beta/rc tags
release = culebrONT.__version__

# -- Project information -----------------------------------------------------
# General information about the project.
project = conf['project']['name']
authors = conf['project']['authors']
date = datetime.now()
copyright = "2019-{year}, J Orjuela (IRD), A Comte (IRD), S Ravel (CIRAD), F Charriat (INRAE), T Vi (IRD, AGI), F Sabot (IRD) and S Cunnac (IRD)".format(year=date.timetuple()[0])

# -- General configuration ---------------------------------------------------
# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.

extensions = [
              'sphinx.ext.viewcode',
              'sphinx.ext.autosectionlabel',
              'sphinx_copybutton',
              "sphinx_rtd_theme",
              'sphinx_click'
              ]
gitlab_url = 'forge.ird.fr'
gitlab_user = 'diade/culebront'
gitlab_repo = 'forge.ird.fr'
gitlab_version = 'master'
display_gitlab = True

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = ['.rst', "md"]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ['_build']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

master_doc = 'index'

# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'sphinx_rtd_theme'
html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
html_theme_options = {
    'analytics_id': 'UA-172723859-1',  #  Provided by Google in your dashboard
    'analytics_anonymize_ip': False,
    'logo_only': False,
    'display_version': True,
    'prev_next_buttons_location': 'bottom',
    'style_external_links': False,
    'vcs_pageview_mode': '',
    'style_nav_header_background': '#2980B9',
    # Toc options
    'collapse_navigation': False,
    'sticky_navigation': True,
    'navigation_depth': 3,
    'includehidden': False,
    'titles_only': False,
}

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
html_title = "CulebrONT"

# A shorter title for the navigation bar.  Default is the same as html_title.
html_short_title = "CulebrONT"

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
html_logo = '_images/culebront_logo.png'

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
html_favicon = '_images/culebront_logo2.png'
global html_context

html_context = {
    "gitlab_host": "forge.ird.fr",
    "display_gitlab": True, # Integrate Gitlab
    "gitlab_user": "diade/culebrONT", # Username
    "gitlab_repo": "culebrONT", # Repo name
    "gitlab_version": "master", # Version
    "conf_py_path": "/docs/source/", # Path in the checkout to the docs root
}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# If false, no index is generated.
html_use_index = True

# If true, the index is split into individual pages for each letter.
html_split_index = False

# If true, links to the reST sources are added to the pages.
# html_show_sourcelink = True

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
html_show_sphinx = False

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
html_show_copyright = True



#!/usr/bin/env python3
from pathlib import Path

# get samples_qmd list
samples_list = snakemake.input.list_sample_qmd
samples_list_for_txt = []
for sample in samples_list:
    file = Path(sample).name
    parent = Path(sample).parent.name
    sample_txt = f'        - {parent}/{file}\n'
    samples_list_for_txt.append(sample_txt)
sample_txt_list = "".join(samples_list_for_txt).rstrip("\n")


quarto_conf_in = Path(snakemake.input.quarto)
with open(quarto_conf_in, "r") as input_conf:
    conf_txt = input_conf.read()

output_dir = snakemake.params.out_dir
conf_txt = conf_txt.replace("OUTPUT_DIR", output_dir)
conf_txt = conf_txt.replace("        SAMPLE", sample_txt_list)

quarto_conf_out = Path(snakemake.output.quarto_conf)
with open(quarto_conf_out, "w") as out_conf:
    out_conf.write(conf_txt)




import pandas as pd
from collections import defaultdict, OrderedDict
from pprint import pprint as pp
import re
from pathlib import Path
version = "0.0.1"
pd.set_option("display.precision", 2)


def replace_all(repls, str):
    """
    Function that take a dictionary and text variable and return text variable with replace 'Key' from dictionary with 'Value'.

    :param repls: a python dictionary
    :type repls: dict()
    :param str: a string where remplace some words
    :type str: str()
    :rtype: str()
    :return: - txt with replace 'Key' of dictionary with 'Value' in the input txt

    Example:
        >>> text =  "i like apples, but pears scare me"
        >>> print(replace_all({"apple": "pear", "pear": "apple"}, text))
        i like pears, but apples scare me
    """
    return re.sub('|'.join(re.escape(key) for key in repls.keys()), lambda k: repls[k.group(0)], str)


def parse_flagstat_multi_report(file):
    """
    Take a list of files, parse the data assuming it's a flagstat file
    Returns csv file
    """
    flagstat_regexes = {
        "total": r"(\d+) \+ (\d+) in total \(QC-passed reads \+ QC-failed reads\)",
        "secondary": r"(\d+) \+ (\d+) secondary",
        # "supplementary": r"(\d+) \+ (\d+) supplementary",
        "duplicates": r"(\d+) \+ (\d+) duplicates",
        # "mapped": r"(\d+) \+ (\d+) mapped \((.+):(.+)\)",
        "paired in sequencing": r"(\d+) \+ (\d+) paired in sequencing",
        # "read1": r"(\d+) \+ (\d+) read1",
        # "read2": r"(\d+) \+ (\d+) read2",
        "properly paired": r"(\d+) \+ (\d+) properly paired \((.+):(.+)\)",
        # "with itself and mate mapped": r"(\d+) \+ (\d+) with itself and mate mapped",
        "singletons": r"(\d+) \+ (\d+) singletons \((.+):(.+)\)",
        "mate mapped to a diff chr": r"(\d+) \+ (\d+) with mate mapped to a different chr",
        "mate mapped to a diff chr (mapQ >= 5)": r"(\d+) \+ (\d+) with mate mapped to a different chr mapQ>=5\)",
    }
    parsed_data = {}
    # re_groups = ["passed", "failed", "passed_pct", "failed_pct"]
    re_groups = ["passed", "failed"]

    file_str = Path(file).open("r").read()
    for k, r in flagstat_regexes.items():
        r_search = re.search(r, file_str, re.MULTILINE)
        if r_search:
            for i, j in enumerate(re_groups):
                try:
                    key = "{}_{}".format(k, j)
                    val = r_search.group(i + 1).strip("% ")
                    parsed_data[key] = float(val) if ("." in val) else int(val)
                except IndexError:
                    pass  # Not all regexes have percentages
                except ValueError:
                    parsed_data[key] = float("nan")
    # Work out the total read count
    try:
        parsed_data["(%) mapped_pass"] = f'{(parsed_data["total_passed"]/parsed_data["total"])*100:.2f}%'
        parsed_data["(%) properly_paired"] = f'{(parsed_data["properly paired_passed"]/parsed_data["total"])*100:.2f}%'
    except KeyError as e:
        print(e)
        pass
    return parsed_data


def main():
    assembly_list = snakemake.params.assembly_list
    quality_list = snakemake.params.quality_list
    out_repository = snakemake.params.out_dir
    out_stats = snakemake.output.stat
    summary_flagstats = snakemake.input.summary

    dico_flagstats_time = defaultdict(OrderedDict)

    for flagstats_file in sorted(summary_flagstats):
        flagstats_file_path = Path(flagstats_file)
        assembler, step, _ = flagstats_file_path.stem.split("-")

        step = replace_all({"STEP_":"","_STARTFIXED":""}, step)
        dico_flagstats_file = parse_flagstat_multi_report(flagstats_file)

        dico_flagstats_time[assembler][step] = dico_flagstats_file

    dataframe_mapping_stats = pd.DataFrame.from_dict(dico_flagstats_time)
    dataframe_mapping_stats = dataframe_mapping_stats.T.stack().apply(pd.Series)
    dataframe_mapping_stats.drop(dataframe_mapping_stats.filter(regex='_failed').columns, axis=1, inplace=True)
    dataframe_mapping_stats.columns = [name.replace("_passed", "") for name in dataframe_mapping_stats.columns]
    dataframe_mapping_stats.reset_index(inplace=True)
    try:
        dataframe_mapping_stats.rename({"level_0":'Assembler',"level_1":'Steps' }, axis='columns', inplace=True, errors="raise")
    except KeyError as e:
        print(dataframe_mapping_stats)

    with open(out_stats, "w") as out_csv_file:
        with pd.option_context('display.float_format', '{:0.2f}'.format):
            print(f"Library size:\n{dataframe_mapping_stats}\n")
            dataframe_mapping_stats.to_csv(out_csv_file, index=False, header=True)


if __name__ == '__main__':
    main()
